@@ -25,9 +25,15 @@ Usage example:
 """
 
 from .queries import Query, where
-from .storages import Storage, JSONStorage
+from .storages import Storage, JSONStorage, EncryptedJSONStorage, MemoryStorage
+from .modifier import Modifier
 from .database import TinyDB
 from .version import __version__
 from .table import BaseID, BaseDocument
 
-__all__ = ('TinyDB', 'Storage', 'JSONStorage', 'Query', 'where', "BaseID", "BaseDocument")
+__all__ = (
+    "TinyDB", "Storage", "JSONStorage",
+    "EncryptedJSONStorage", "MemoryStorage",
+    "Query", "where", "BaseID", "BaseDocument",
+    "Modifier",
+    )
